@@ -26,12 +26,39 @@
 #include <glib.h>
 #include "totem_internal.h"
 
+/* A macro to call functions synchronously or asynchronously, depending on whether
+ * we're parsing sync or async. This is necessary because when doing a sync parse,
+ * the main loop isn't iterated, and so any signals emitted in idle functions (a requirement
+ * of an async parse) aren't emitted until the sync parsing function returns, which
+ * is less than ideal. We therefore want those idle functions to be called synchronously
+ * when parsing sync, and with g_idle_add() when parsing async.
+ *
+ * Whether we're parsing sync or async is determined by whether we're in a thread. If (for
+ * whatever reason), we're parsing async -- but not in a thread -- this will work out fine
+ * anyway, since signal emission will consequently happen in the main thread.
+ *
+ * We determine if we're in the main thread by comparing the GThread pointer of the current
+ * thread to a stored GThread pointer known to be from the main thread
+ * (TotemPlParser->priv->main_thread).
+ *
+ * @p: a #TotemPlParser
+ * @c: callback (as if for g_idle_add())
+ * @d: callback data
+ */
+#define CALL_ASYNC(p, c, d) {				\
+	if (g_thread_self () == p->priv->main_thread)	\
+		c (d);					\
+	else						\
+		g_idle_add ((GSourceFunc) c, d);	\
+}
+
 #ifndef TOTEM_PL_PARSER_MINI
 #include "totem-pl-parser.h"
 #include <glib-object.h>
 #include <gio/gio.h>
 #include <gio/gio.h>
 #include <string.h>
+#include "xmlparser.h"
 #else
 #include "totem-pl-parser-mini.h"
 #endif /* !TOTEM_PL_PARSER_MINI */
@@ -55,7 +82,7 @@
 
 #ifndef TOTEM_PL_PARSER_MINI
 #define DEBUG(file, x) {					\
-	if (parser->priv->debug) {				\
+	if (totem_pl_parser_is_debugging_enabled (parser)) {	\
 		if (file != NULL) {				\
 			char *uri;				\
 								\
@@ -69,29 +96,24 @@
 	}							\
 }
 #else
-#define DEBUG(x) { if (parser->priv->debug) x; }
+#define DEBUG(x) { if (totem_pl_parser_is_debugging_enabled (parser)) x; }
 #endif
 
-struct TotemPlParserPrivate
-{
-	GList *ignore_schemes;
-	GList *ignore_mimetypes;
-
+typedef struct {
 	guint recurse_level;
 	guint fallback : 1;
 	guint recurse : 1;
-	guint debug : 1;
 	guint force : 1;
 	guint disable_unsafe : 1;
-};
+} TotemPlParseData;
 
 #ifndef TOTEM_PL_PARSER_MINI
-char *totem_pl_parser_read_ini_line_string	(char **lines, const char *key,
-						 gboolean dos_mode);
+char *totem_pl_parser_read_ini_line_string	(char **lines, const char *key);
 int   totem_pl_parser_read_ini_line_int		(char **lines, const char *key);
 char *totem_pl_parser_read_ini_line_string_with_sep (char **lines, const char *key,
-						     gboolean dos_mode, const char *sep);
-char *totem_pl_parser_base_url			(GFile *file);
+						     const char *sep);
+gboolean totem_pl_parser_is_debugging_enabled	(TotemPlParser *parser);
+char *totem_pl_parser_base_uri			(GFile *file);
 void totem_pl_parser_playlist_end		(TotemPlParser *parser,
 						 const char *playlist_title);
 int totem_pl_parser_num_entries			(TotemPlParser *parser,
@@ -110,21 +132,25 @@ gboolean totem_pl_parser_write_buffer		(GOutputStream *stream,
 						 GError **error);
 char * totem_pl_parser_relative			(GFile *output,
 						 const char *filepath);
+char * totem_pl_parser_resolve_uri		(GFile *base_gfile,
+						 const char *relative_uri);
 TotemPlParserResult totem_pl_parser_parse_internal (TotemPlParser *parser,
 						    GFile *file,
-						    GFile *base_file);
-void totem_pl_parser_add_one_url		(TotemPlParser *parser,
-						 const char *url,
+						    GFile *base_file,
+						    TotemPlParseData *parse_data);
+void totem_pl_parser_add_one_uri		(TotemPlParser *parser,
+						 const char *uri,
 						 const char *title);
 void totem_pl_parser_add_one_file		(TotemPlParser *parser,
 						 GFile *file,
 						 const char *title);
-void totem_pl_parser_add_url			(TotemPlParser *parser,
+void totem_pl_parser_add_uri			(TotemPlParser *parser,
 						 const char *first_property_name,
 						 ...);
 gboolean totem_pl_parser_ignore			(TotemPlParser *parser,
-						 const char *url);
-void totem_pl_parser_cleanup_xml		(char *string);
+						 const char *uri);
+xml_node_t * totem_pl_parser_parse_xml_relaxed	(char *contents,
+						 gsize size);
 
 #endif /* !TOTEM_PL_PARSER_MINI */
 
